from __future__ import print_function

import logging
import sys

import os
import leip
import hashlib
from mad2.util import get_all_mad_files

lg = logging.getLogger(__name__)


def _get_qdhash(filename):
    """
    Provde a quick & dirty hash

    this is by no means secure, but quick for very large files, and as long
    as one does not try to create duplicate hashes, the chance is still very
    slim that a duplicate will arise
    """
    sha1sum = hashlib.sha1()
    filesize = os.stat(filename).st_size
    if filesize < 10000:
        with open(filename) as F:
            sha1sum.update(F.read().encode())
    else:
        with open(filename) as F:
            sha1sum.update(F.read(2000))
            F.seek(int(filesize * 0.4))
            sha1sum.update(F.read(2000))
            F.seek(-2000, 2)
            sha1sum.update(F.read())
    return sha1sum.hexdigest()

def hashit(hasher, filename):
    """
    Provde a quick & dirty hash

    this is by no means secure, but quick for very large files, and as long
    as one does not try to create duplicate hashes, the chance is still very
    slim that a duplicate will arise
    """
    h = hasher()
    with open(filename) as F:
        h.update(F.read())
    return h.hexdigest()


@leip.arg('-f', '--force', action='store_true', help='apply force')
@leip.arg('file', nargs='*')
@leip.command
def md5(app, args):
    """
    Calculate a checksum
    """
    for madfile in get_all_mad_files(app, args):
        if not args.force and 'md5' in madfile.mad:
            #exists - and not forcing
            lg.warning("Skipping md5 checksum - exists")
            continue
        lg.info("Processing %s for md5sum" % madfile.filename)
        cs = hashit(hashlib.md5, madfile.filename)
        madfile.mad.hash.md5 = cs
        madfile.save()

@leip.arg('-f', '--force', action='store_true', help='apply force')
@leip.arg('file', nargs='*')
@leip.command
def sha1(app, args):
    """
    Calculate a sha1 checksum
    """
    for madfile in get_all_mad_files(app, args):
        if not args.force and 'sha1' in madfile.mad:
            #exists - and not forcing
            lg.warning("Skipping sha1 checksum - exists")
            continue
        cs = hashit(hashlib.sha1, madfile.filename)
        madfile.mad.hash.sha1 = cs
        madfile.save()

@leip.arg('-f', '--force', action='store_true', help='apply force')
@leip.arg('file', nargs='*')
@leip.command
def qdhash(app, args):
    """
    Calculate a sha1 checksum
    """
    for madfile in get_all_mad_files(app, args):
        if not args.force and 'qdhash' in madfile.mad:
            #exists - and not forcing
            lg.warning("Skipping sha1 checksum - exists")
            continue
        cs = _get_qdhash(madfile.filename)
        madfile.mad.hash.qdhash = cs
        madfile.save()




import os
import Yaco
import copy
import logging

from jinja2 import Template

lg = logging.getLogger(__name__)


def dummy_hook_method(*args, **kw):
    return None


class MadFile(object):
    """

    """
    #def __init__(self, filename, hash_func=None):
    def __init__(self, filename, hook_method=dummy_hook_method):

        if filename[-4:] == '.mad':
            filename = filename[:-4]

        self.mad = Yaco.Yaco()
        self.otf = Yaco.Yaco()  # on the fly stuff

        self.otf.filename = filename
        self.otf.madname = self.otf.filename + '.mad'

        self.hook_method = hook_method
        self.load()

    def __getitem__(self, item):
        if item in self.mad:
            return self.mad[item]
        elif item in self.otf:
            return self.otf[item]
        raise KeyError()

    __getattr__ = __getitem__

    def data(self, on_top_of={}):
        """Render data into a dict like format
        """
        if isinstance(on_top_of, Yaco.Yaco) or \
            isinstance(on_top_of, Yaco.PolyYaco):
            data = on_top_of.simple()

        data.update(self.otf.simple())
        data.update(self.mad.simple())
        return data

    def render(self, text, base):

        if isinstance(base, Yaco.PolyYaco):
            data = base.merge()
        else:
            data = copy.copy(base)

        data.update(self.mad)
        data['madname'] = self.madname
        data['filename'] = self.filename

        rendered = text
        last = rendered
        iteration = 0
        while '{{' in rendered or '{%' in rendered:
            if iteration > 0 and rendered == last:
                #no improvement
                break
            template = Template(rendered)
            rendered = template.render(data)
        return rendered

    def load(self):
        if os.path.exists(self.madname):
            lg.debug("loading madfile {0}".format(self.madname))
            self.mad.load(self.madname)
            self.hook_method('madfile_load', self)

    def save(self):
        self.hook_method('madfile_save', self)
        self.mad.save(self.madname)

    def pretty(self):
        return self.mad.pretty()



# -*- coding: utf-8 -*-
from __future__ import print_function,  unicode_literals
import logging
import os
import sys
import subprocess
import tempfile

import leip
import Yaco

import mad2.ui
from mad2.util import  get_mad_file, get_all_mad_files

lg = logging.getLogger(__name__)

def dispatch():
    """
    Run the app - this is the actual entry point
    """
    app.run()

##
## define Mad commands
##

@leip.arg('file', nargs='?')
@leip.arg('key', help='key to set')
@leip.command
def edit(app, args):
    """
    Edit a key in a full screen editor
    """
    key = args.key
    editor = os.environ.get('EDITOR','vim')

    if args.file:
        madfile = get_mad_file(app, args.file)
        default = madfile.mad.get(key, "")
    else:
        #if no file is defined, use the configuration
        default = app.conf.get(key, "")

    #write default value to a temp file, and start the editor
    tmp_file = tempfile.NamedTemporaryFile('wb', delete=False)
    if default:
        tmp_file.write(default)
    tmp_file.close()
    subprocess.call([editor, tmp_file.name])

    #read tmp file
    with open(tmp_file.name, 'r') as F:
        #removing trailing space
        val = F.read().rstrip()

    if args.file:
        madfile.mad[key] = val
        madfile.save()
    else:
        app.conf[key] = val
        app.conf.save()

    os.unlink(tmp_file.name)


@leip.arg('-f', '--force', action='store_true', help='apply force')
@leip.arg('file', nargs='*')
@leip.arg('value', help='value to set')
@leip.arg('key', help='key to set')
@leip.command
def set(app, args):

    key = args.key
    val = args.value

    madfiles = list(get_all_mad_files(app, args))

    if val == '-':
        if len(madfiles) == 1:
            data = madfiles[0].data(app.conf)
            default = madfiles[0].mad.get(key, "")
        else:
            data = app.conf.simple()

        val = mad2.ui.askUser(key, default, data)

    if len(madfiles) == 0:
        #apply conf to the local user config if no madfiles are defined
        app.conf[key] = val
        app.conf.save()
        return

    for madfile in madfiles:
        list_mode = False
        if key[0] == '+':
            list_mode = True
            key = key[1:]

        keywords = app.conf.keywords
        if not args.force and not key in keywords:
            print("invalid key: {0} (use -f?)".format(key))
            sys.exit(-1)

        keyinfo = keywords[key]
        keytype = keyinfo.get('type', 'str')

        if list_mode and keyinfo.cardinality == '1':
            print("Cardinality == 1 - no lists!")
            sys.exit(-1)
        elif keyinfo.cardinality == '+':
            list_mode = True

        if keytype == 'restricted' and \
                not val in keyinfo.allowed:
            print("Value '{0}' not allowed".format(val))
            sys.exit(-1)

        if list_mode:
            if not key in madfile.mad:
                oldval = []
            else:
                oldval = madfile.mad[key]
                if not isinstance(oldval, list):
                    oldval = [oldval]
            madfile.mad[key] = oldval + [val]
        else:
            #not listmode
            madfile.mad[key] = val

        madfile.save()


##
## define show
##
@leip.arg('-a', '--all', action='store_true')
@leip.arg('file', nargs='*')
@leip.command
def show(app, args):
    for madfile in get_all_mad_files(app, args):
        d = madfile.mad.copy()
        d.update(madfile.otf)
        print('---')
        print(d.pretty().strip())

##
## define show
##
@leip.arg('file', nargs="+")
@leip.arg('key')
@leip.command
def unset(app, args):
    lg.debug("unsetting: %s".format(args.key))
    key = args.key
    keyinfo = keywords[key]
    if keyinfo.cardinality =='+':
        print("Not implemented - unsetting keys with cardinality > 1")
        sys.exit(-1)
    for madfile in get_all_mad_files(app, args):
        if args.key in madfile.mad:
            del(madfile.mad[args.key])
        madfile.save()

##
## define find
##
@leip.arg('dir', default='.', help='directory to search from')
@leip.command
def find(app, args):
    lg.info("searching dir %s" % args.dir)


##
## Instantiate the app and discover hooks & commands
##

import pkg_resources
base_config = pkg_resources.resource_string('mad2', 'etc/mad.config')

#trail of config files???
config_files = [
    ('system', '/etc/mad.config'),
    ('user', '~/.config/mad/mad.config')]

path = os.getcwd()
config_no = 0
xtra_config = []
while path:
    config_no += 1
    xtra_config.append(('dd{0}'.format(config_no), os.path.join(path, '.mad')))
    path = path.rsplit(os.sep, 1)[0]

config_files.extend(list(reversed(xtra_config)))
app = leip.app(name='mad', set_name=None, base_config = base_config,
               config_files = config_files)

#discover hooks in this module!
app.discover(globals())



from __future__ import print_function

import re
import logging
import sys

from Yaco import Yaco
import leip

from mad2.util import  get_filenames, get_all_mad_files

lg = logging.getLogger(__name__)

def _getter(o, k):
    assert(isinstance(o, Yaco))
    if '.' in k:
        k1, k2 = k.split('.', 1)
        return _getter(o[k1], k2)
    else:
        return o[k]


@leip.arg('-s', '--sep',  help='separator', default='\t')
@leip.arg('key', help='keys to display', nargs='+')
@leip.command
def table(app, args):
    """
    Create a table
    """
    print('#' + args.sep.join(map(str, args.key)))
    for madfile in get_all_mad_files(app, args):
        values = []
        for k in args.key:
            vv = _getter(madfile.mad, k)
            if str(vv) == '{}':
                vv = _getter(madfile.otf, k)
            if str(vv) == '{}':
                vv = ""
            values.append(vv)

        print(args.sep.join(map(str, values)))










from __future__ import print_function
import os
import logging
import socket
import leip
from pwd import getpwuid
import hashlib
from mad2.util import get_all_mad_files

lg = logging.getLogger(__name__)

@leip.hook("madfile_load")
def onthefly(app, madfile):
    lg.debug("running onthelfy")
    madfile.otf.fullpath = os.path.abspath(madfile.filename)
    lg.debug("get fqdn")
    madfile.otf.host = socket.gethostname()
    filestat = os.stat(madfile.filename)
    madfile.otf.filesize = filestat.st_size
    userinfo = getpwuid(filestat.st_uid)
    madfile.otf.userid = userinfo.pw_name
    madfile.otf.username = userinfo.pw_gecos
    lg.debug("finished onthefly")

@leip.arg('file', nargs='*')
@leip.command
def qdhash(app, args):
    for madfile in get_all_mad_files(app, args):
        print("{0}  {1}".format(qd_hash(madfile.filename), madfile.filename))


from __future__ import print_function

import logging
from pymongo import MongoClient
from bson.objectid import ObjectId
import leip
import copy
import datetime
import pprint
from mad2.util import get_all_mad_files


lg = logging.getLogger(__name__)
#lg.setLevel(logging.DEBUG)

@leip.hook("madfile_save")
def store_in_mongodb(app, madfile):
    lg.debug("running store_in_mongodb")
    mng_mad, mng_hist = _get_mng_collection(app)
    if 'mongoid' in madfile.mad:
        mongoid = madfile.mad.mongoid
        hist_rec = mng_mad.find_one({'_id': ObjectId(mongoid)})
        #keep historical record
        del(hist_rec["_id"])
        hist_rec['mongoid'] = mongoid
        hist_rec['date'] = datetime.datetime.now()
        mng_hist.insert(hist_rec)
        #store record
        newrec = madfile.data()
        del(newrec['mongoid'])
        newrec['_id'] = ObjectId(mongoid)
        mng_mad.save(newrec)
    else:
        mongoid = mng_mad.insert(madfile.data())
        lg.debug("Updating mongo record {0}".format(mongoid))
        madfile.mad.mongoid = str(mongoid)

def _get_mng_collection(app):
    host = app.conf.plugin.mongo.host
    port = app.conf.plugin.mongo.get('port', 27017)
    client = MongoClient(host, port)
    return client.mad.mad, client.mad.history

@leip.arg('file', nargs="+")
@leip.command
def mongo_show(app, args):
    """
    Show the associated mongodb record
    """
    mng_mad, mng_hist = _get_mng_collection(app)
    for madfile in get_all_mad_files(app, args):
        if 'mongoid' in madfile.mad:
            mongoid = madfile.mad.mongoid
            rec = mng_mad.find_one({'_id': ObjectId(mongoid)})
            #print(madfile.filename)
            for key in rec:
                print('- {0}: {1}'.format(key, rec[key]))




#!/usr/bin/env python

from setuptools import setup, find_packages

DESCRIPTION = """
file metadata tagger
"""

entry_points = {
    'console_scripts': [
        'mad = mad2.cli.main:dispatch'
        ]}

setup(name='mad2',
      version='0.0.10',
      description=DESCRIPTION,
      author='Mark Fiers',
      author_email='mark.fiers42@gmail.com',
      entry_points = entry_points,
      include_package_data=True,
      url='http://mfiers.github.com/mad2',
      packages=find_packages(),
      requires=['Leip',
                'Yaco',
                'xlrd',
                'Jinja2'
                ],
      classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        ]

     )



from __future__ import print_function

import logging

import leip
from mad2.util import get_mad_file

lg = logging.getLogger(__name__)

@leip.arg('file')
@leip.arg('key', help='key to filter on')
@leip.commandName('print')
def mad_print(app, args):
    """
    print a single value from a single file
    """
    madfile = get_mad_file(app, args.file)
    print(madfile.render(madfile.mad[args.key], app.conf))

